@@ -1,16 +1,14 @@
 package WWW::Crawler::Mojo;
 use strict;
 use warnings;
-use 5.010;
 use Mojo::Base 'Mojo::EventEmitter';
 use WWW::Crawler::Mojo::Job;
 use WWW::Crawler::Mojo::Queue::Memory;
 use WWW::Crawler::Mojo::UserAgent;
 use WWW::Crawler::Mojo::ScraperUtil qw{
   collect_urls_css html_handlers resolve_href decoded_body};
-use Mojo::Util 'deprecated';
 use Mojo::Message::Request;
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 has clock_speed       => 0.25;
 has max_conn          => 1;
@@ -26,7 +24,7 @@ sub crawl {
 
   $self->init;
 
-  die 'No job is given' if (!$self->queue->length);
+  die 'No job is given' unless ($self->queue->length);
 
   $self->emit('start');
 
@@ -53,15 +51,13 @@ sub init {
       my $queue = $self->queue;
 
       if (!$queue->length) {
-        $self->emit('empty') if (!$self->ua->active_conn);
-        return;
-      }
-      if ( $self->ua->active_conn >= $self->max_conn
-        || $self->ua->active_host($queue->next->url)
-        >= $self->max_conn_per_host)
-      {
+        $self->emit('empty') unless ($self->ua->active_conn);
         return;
       }
+      return if $self->ua->active_conn >= $self->max_conn;
+      return
+        if $self->ua->active_host($queue->next->url)
+        >= $self->max_conn_per_host;
       $self->process_job($queue->dequeue);
     }
   );
@@ -82,18 +78,22 @@ sub init {
 sub process_job {
   my ($self, $job) = @_;
 
+  my $tx = $self->ua->build_tx($job->method
+      || 'get' => $job->url => $job->tx_params);
+
+  $self->emit('req', $job, $tx->req);
+
   $self->ua->start(
-    $self->ua->build_tx($job->method || 'get' => $job->url => $job->tx_params),
-    sub {
+    $tx => sub {
       my ($ua, $tx) = @_;
 
       $job->redirect(_urls_redirect($tx));
 
       my $res = $tx->res;
 
-      if (!$res->code) {
-        $self->emit('error',
-          ($res->error) ? $res->error->{message} : 'Unknown error', $job);
+      unless ($res->code) {
+        my $msg = $res->error ? $res->error->{message} : 'Unknown error';
+        $self->emit('error', $msg, $job);
         return;
       }
 
@@ -117,9 +117,6 @@ EOF
 }
 
 sub scrape {
-  die 'scrape method with 4 argments has been removed'
-    if (scalar @_ == 5 || ref $_[3] eq 'CODE');
-
   my ($self, $res, $job, $contexts) = @_;
   my @ret;
 
@@ -299,6 +296,17 @@ Name of crawler for User-Agent header.
 L<WWW::Crawler::Mojo> inherits all events from L<Mojo::EventEmitter> and
 implements the following new ones.
 
+=head2 req
+
+Emitted right before crawler perform request to servers. The callback takes 3
+arguments.
+
+    $bot->on(req => sub {
+        my ($bot, $job, $req) = @_;
+
+        # DO NOTHING
+    });
+
 =head2 res
 
 Emitted when crawler got response from server. The callback takes 4 arguments.
